@@ -6,18 +6,23 @@ import xarray as xr
 from ..echodata import EchoData
 from ..echodata.simrad import retrieve_correct_beam_group
 from ..utils.log import _init_logger
-from .cal_params import get_cal_params_EK
+from .cal_params import _get_interp_da, get_cal_params_EK
 from .calibrate_base import CalibrateBase
+from .ecs import conform_channel_order, ecs_ds2dict, ecs_ev2ep
 from .ek80_complex import compress_pulse, get_filter_coeff, get_tau_effective, get_transmit_signal
-from .env_params import get_env_params_EK60, get_env_params_EK80
+from .env_params import get_env_params_EK
 from .range import compute_range_EK, range_mod_TVG_EK
 
 logger = _init_logger(__name__)
 
 
 class CalibrateEK(CalibrateBase):
-    def __init__(self, echodata: EchoData, env_params, cal_params):
-        super().__init__(echodata, env_params, cal_params)
+    def __init__(self, echodata: EchoData, env_params, cal_params, ecs_file, **kwargs):
+        super().__init__(echodata, env_params, cal_params, ecs_file)
+
+        self.ed_beam_group = None  # will be assigned in child class
+
+        self.ed_beam_group = None  # will be assigned in child class
 
     def compute_echo_range(self, chan_sel: xr.DataArray = None):
         """
@@ -36,7 +41,7 @@ class CalibrateEK(CalibrateBase):
             chan_sel=chan_sel,
         )
 
-    def _cal_power_samples(self, cal_type: str, power_ed_group: str = None) -> xr.Dataset:
+    def _cal_power_samples(self, cal_type: str) -> xr.Dataset:
         """Calibrate power data from EK60 and EK80.
 
         Parameters
@@ -44,8 +49,6 @@ class CalibrateEK(CalibrateBase):
         cal_type: str
             'Sv' for calculating volume backscattering strength, or
             'TS' for calculating target strength
-        power_ed_group:
-            The ``EchoData`` beam group path containing the power data
 
         Returns
         -------
@@ -53,7 +56,7 @@ class CalibrateEK(CalibrateBase):
             The calibrated dataset containing Sv or TS
         """
         # Select source of backscatter data
-        beam = self.echodata[power_ed_group]
+        beam = self.echodata[self.ed_beam_group]
 
         # Derived params
         wavelength = self.env_params["sound_speed"] / beam["frequency_nominal"]  # wavelength
@@ -63,7 +66,7 @@ class CalibrateEK(CalibrateBase):
         sound_speed = self.env_params["sound_speed"]
         absorption = self.env_params["sound_absorption"]
         tvg_mod_range = range_mod_TVG_EK(
-            self.echodata, self.ed_group, self.range_meter, sound_speed
+            self.echodata, self.ed_beam_group, self.range_meter, sound_speed
         )
         tvg_mod_range = tvg_mod_range.where(tvg_mod_range > 0, np.nan)
 
@@ -127,31 +130,46 @@ class CalibrateEK(CalibrateBase):
 
 
 class CalibrateEK60(CalibrateEK):
-    def __init__(self, echodata, env_params, cal_params, **kwargs):
-        super().__init__(echodata, env_params, cal_params)
+    def __init__(self, echodata: EchoData, env_params, cal_params, ecs_file, **kwargs):
+        super().__init__(echodata, env_params, cal_params, ecs_file)
 
-        # Set sonar_type
+        # Set sonar_type and waveform/encode mode
         self.sonar_type = "EK60"
 
-        # Get env_params
-        self.env_params = get_env_params_EK60(echodata=echodata, user_env_dict=self.env_params)
+        # Set cal type
         self.waveform_mode = "CW"
         self.encode_mode = "power"
 
-        # Compute range
-        self.compute_echo_range()
-
-        # Get the right ed_group for CW power samples
-        self.ed_group = retrieve_correct_beam_group(
+        # Get the right ed_beam_group for CW power samples
+        self.ed_beam_group = retrieve_correct_beam_group(
             echodata=self.echodata, waveform_mode=self.waveform_mode, encode_mode=self.encode_mode
         )
 
         # Set the channels to calibrate
         # For EK60 this is all channels
-        self.chan_sel = self.echodata[self.ed_group]["channel"]
-        beam = self.echodata[self.ed_group]
+        self.chan_sel = self.echodata[self.ed_beam_group]["channel"]
+        beam = self.echodata[self.ed_beam_group]
 
-        # Get cal_params
+        # Convert env_params and cal_params if self.ecs_file exists
+        # Note a warning if thrown out in CalibrateBase.__init__
+        # to let user know cal_params and env_params are ignored if ecs_file is provided
+        if self.ecs_file is not None:  # also means self.ecs_dict != {}
+            ds_env_tmp, ds_cal_tmp, _ = ecs_ev2ep(self.ecs_dict, "EK60")
+            self.cal_params = ecs_ds2dict(
+                conform_channel_order(ds_cal_tmp, beam["frequency_nominal"])
+            )
+            self.env_params = ecs_ds2dict(
+                conform_channel_order(ds_env_tmp, beam["frequency_nominal"])
+            )
+
+        # Regardless of the source cal and env params,
+        # go through the same sanitization and organization process
+        self.env_params = get_env_params_EK(
+            sonar_type=self.sonar_type,
+            beam=self.echodata[self.ed_beam_group],
+            env=self.echodata["Environment"],
+            user_dict=self.env_params,
+        )
         self.cal_params = get_cal_params_EK(
             waveform_mode=self.waveform_mode,
             freq_center=beam["frequency_nominal"],
@@ -161,11 +179,14 @@ class CalibrateEK60(CalibrateEK):
             sonar_type=self.sonar_type,
         )
 
+        # Compute range
+        self.compute_echo_range()
+
     def compute_Sv(self, **kwargs):
-        return self._cal_power_samples(cal_type="Sv", power_ed_group=self.ed_group)
+        return self._cal_power_samples(cal_type="Sv")
 
     def compute_TS(self, **kwargs):
-        return self._cal_power_samples(cal_type="TS", power_ed_group=self.ed_group)
+        return self._cal_power_samples(cal_type="TS")
 
 
 class CalibrateEK80(CalibrateEK):
@@ -185,8 +206,17 @@ class CalibrateEK80(CalibrateEK):
         "WBT LF": 93750,
     }
 
-    def __init__(self, echodata, env_params, cal_params, waveform_mode, encode_mode):
-        super().__init__(echodata, env_params, cal_params)
+    def __init__(
+        self,
+        echodata: EchoData,
+        env_params,
+        cal_params,
+        waveform_mode,
+        encode_mode,
+        ecs_file=None,
+        **kwargs,
+    ):
+        super().__init__(echodata, env_params, cal_params, ecs_file)
 
         # Set sonar_type
         self.sonar_type = "EK80"
@@ -197,8 +227,8 @@ class CalibrateEK80(CalibrateEK):
         self.encode_mode = encode_mode
         self.echodata = echodata
 
-        # Get the right ed_group given waveform and encode mode
-        self.ed_group = retrieve_correct_beam_group(
+        # Get the right ed_beam_group given waveform and encode mode
+        self.ed_beam_group = retrieve_correct_beam_group(
             echodata=self.echodata, waveform_mode=self.waveform_mode, encode_mode=self.encode_mode
         )
 
@@ -206,14 +236,14 @@ class CalibrateEK80(CalibrateEK):
         if self.encode_mode == "power":
             # Power sample only possible under CW mode,
             # and all power samples will live in the same group
-            self.chan_sel = self.echodata[self.ed_group]["channel"]
+            self.chan_sel = self.echodata[self.ed_beam_group]["channel"]
         else:
             # Complex samples can be CW or BB, so select based on waveform mode
-            chan_dict = self._get_chan_dict(self.echodata[self.ed_group])
+            chan_dict = self._get_chan_dict(self.echodata[self.ed_beam_group])
             self.chan_sel = chan_dict[self.waveform_mode]
 
         # Subset of the right Sonar/Beam_groupX group given the selected channels
-        beam = self.echodata[self.ed_group].sel(channel=self.chan_sel)
+        beam = self.echodata[self.ed_beam_group].sel(channel=self.chan_sel)
 
         # Use center frequency if in BB mode, else use nominal channel frequency
         if self.waveform_mode == "BB":
@@ -225,12 +255,39 @@ class CalibrateEK80(CalibrateEK):
             # use nominal channel frequency for CW pulse
             self.freq_center = beam["frequency_nominal"].sel(channel=self.chan_sel)
 
+        # Convert env_params and cal_params if self.ecs_file exists
+        # Note a warning if thrown out in CalibrateBase.__init__
+        # to let user know cal_params and env_params are ignored if ecs_file is provided
+        if self.ecs_file is not None:  # also means self.ecs_dict != {}
+            ds_env, ds_cal_NB, ds_cal_BB = ecs_ev2ep(self.ecs_dict, "EK80")
+            self.env_params = ecs_ds2dict(
+                conform_channel_order(ds_env, beam["frequency_nominal"].sel(channel=self.chan_sel))
+            )
+            ds_cal_BB = conform_channel_order(
+                ds_cal_BB, beam["frequency_nominal"].sel(channel=self.chan_sel)
+            )
+            ds_cal_NB = self._scale_ecs_cal_params_NB(
+                conform_channel_order(
+                    ds_cal_NB, beam["frequency_nominal"].sel(channel=self.chan_sel)
+                ),
+                beam,
+            )
+            cal_params_dict = ecs_ds2dict(ds_cal_NB)
+
+            if ds_cal_BB is not None:
+                # get_cal_params_EK fill in empty params at param level, not channel level,
+                # so need to do freq-dep interpolation here
+                self.cal_params = self._assimilate_ecs_cal_params(cal_params_dict, ds_cal_BB)
+            else:
+                self.cal_params = cal_params_dict
+
         # Get env_params: depends on waveform mode
-        self.env_params = get_env_params_EK80(
-            echodata=echodata,
+        self.env_params = get_env_params_EK(
+            sonar_type=self.sonar_type,
+            beam=beam,
+            env=self.echodata["Environment"],
+            user_dict=self.env_params,
             freq=self.freq_center,
-            user_env_dict=env_params,
-            ed_group=self.ed_group,
         )
 
         # Get cal_params: depends on waveform and encode mode
@@ -240,6 +297,7 @@ class CalibrateEK80(CalibrateEK):
             beam=beam,  # already subset above
             vend=self.echodata["Vendor_specific"].sel(channel=self.chan_sel),
             user_dict=self.cal_params,
+            sonar_type="EK80",
         )
 
         # Compute echo range in meters
@@ -269,6 +327,70 @@ class CalibrateEK80(CalibrateEK):
             # All channels are CW
             return {"BB": None, "CW": beam.channel}
 
+    def _scale_ecs_cal_params_NB(self, ds_cal_NB: xr.Dataset, beam: xr.Dataset) -> xr.Dataset:
+        """
+        Scale narrowband parameters based on center frequency of each ping
+        with respect to channel nominal frequency.
+        """
+        for p in ds_cal_NB:
+            if p in ["angle_sensitivity_alongship", "angle_sensitivity_athwartship"]:
+                ds_cal_NB[p] = ds_cal_NB[p] * self.freq_center / beam["frequency_nominal"]
+            elif p in ["beamwidth_alongship", "beamwidth_athwartship"]:
+                ds_cal_NB[p] = ds_cal_NB[p] * beam["frequency_nominal"] / self.freq_center
+            elif p == "equivalent_beam_angle":
+                ds_cal_NB[p] = ds_cal_NB[p] + 20 * np.log10(
+                    beam["frequency_nominal"] / self.freq_center
+                )
+        return ds_cal_NB
+
+    def _assimilate_ecs_cal_params(self, cal_params_dict: Dict, ds_cal_BB: xr.Dataset):
+        """
+        Combine narrowband and broadband parameters derived from ECS.
+        """
+        if ds_cal_BB is not None:
+            ds_cal_BB = ds_cal_BB.rename({"channel": "cal_channel_id"})
+
+            for p in ds_cal_BB.data_vars:
+                # For parameters where there is frequency-dependent values,
+                # the corresponding narrowband (CW mode) values should exist for all channels
+                if not np.all(
+                    [
+                        ch in cal_params_dict[p]["channel"].values
+                        for ch in ds_cal_BB["cal_channel_id"].values
+                    ]
+                ):
+                    raise ValueError(
+                        f"Narrowband (CW mode) parameter {p} should exist "
+                        "for all channels with frequency-dependent parameter values."
+                    )
+
+                # Assemble parameter data array with all channels
+                # Either interpolate or pull from narrowband input
+                if "ping_time" in cal_params_dict[p].coords:
+                    ds_cal_BB[p] = _get_interp_da(
+                        da_param=ds_cal_BB[p],  # freq-dep xr.DataArray
+                        freq_center=self.freq_center,
+                        alternative=cal_params_dict[p],
+                    )
+                else:
+                    ds_cal_BB[p] = _get_interp_da(
+                        da_param=ds_cal_BB[p],  # freq-dep xr.DataArray
+                        freq_center=self.freq_center,
+                        alternative=cal_params_dict[p].expand_dims(
+                            dim={"ping_time": self.freq_center["ping_time"].size}, axis=1
+                        ),
+                    )
+
+            # Keep only 'channel' and 'ping_time' coorindates
+            ds_cal_BB = ds_cal_BB.drop_dims(["cal_frequency", "cal_channel_id"])
+
+            # Substitute params in narrowband dict
+            return dict(cal_params_dict, **ecs_ds2dict(ds_cal_BB))
+
+        else:
+            # Do nothing if ds_cal_BB is None
+            return cal_params_dict
+
     def _get_power_from_complex(
         self,
         beam: xr.Dataset,
@@ -333,7 +455,7 @@ class CalibrateEK80(CalibrateEK):
 
         return B_theta_phi_m
 
-    def _cal_complex_samples(self, cal_type: str, complex_ed_group: str) -> xr.Dataset:
+    def _cal_complex_samples(self, cal_type: str) -> xr.Dataset:
         """Calibrate complex data from EK80.
 
         Parameters
@@ -341,8 +463,6 @@ class CalibrateEK80(CalibrateEK):
         cal_type : str
             'Sv' for calculating volume backscattering strength, or
             'TS' for calculating target strength
-        complex_ed_group : str
-            The ``EchoData`` beam group path containing complex data
 
         Returns
         -------
@@ -350,7 +470,7 @@ class CalibrateEK80(CalibrateEK):
             The calibrated dataset containing Sv or TS
         """
         # Select source of backscatter data
-        beam = self.echodata[complex_ed_group].sel(channel=self.chan_sel)
+        beam = self.echodata[self.ed_beam_group].sel(channel=self.chan_sel)
         vend = self.echodata["Vendor_specific"].sel(channel=self.chan_sel)
 
         # Get transmit signal
@@ -376,7 +496,9 @@ class CalibrateEK80(CalibrateEK):
         transmit_power = beam["transmit_power"]
 
         # TVG compensation with modified range
-        tvg_mod_range = range_mod_TVG_EK(self.echodata, self.ed_group, range_meter, sound_speed)
+        tvg_mod_range = range_mod_TVG_EK(
+            self.echodata, self.ed_beam_group, range_meter, sound_speed
+        )
         tvg_mod_range = tvg_mod_range.where(tvg_mod_range > 0, np.nan)
 
         spreading_loss = 20 * np.log10(tvg_mod_range)
@@ -476,10 +598,10 @@ class CalibrateEK80(CalibrateEK):
 
         if flag_complex:
             # Complex samples can be BB or CW
-            ds_cal = self._cal_complex_samples(cal_type=cal_type, complex_ed_group=self.ed_group)
+            ds_cal = self._cal_complex_samples(cal_type=cal_type)
         else:
             # Power samples only make sense for CW mode data
-            ds_cal = self._cal_power_samples(cal_type=cal_type, power_ed_group=self.ed_group)
+            ds_cal = self._cal_power_samples(cal_type=cal_type)
 
         return ds_cal
 
