@@ -9,6 +9,7 @@ from ..calibrate.ek80_complex import get_filter_coeff
 from ..echodata import EchoData
 from ..echodata.simrad import retrieve_correct_beam_group
 from ..utils.io import validate_source_ds_da
+from ..utils.prov import add_processing_level
 from .split_beam_angle import add_angle_to_ds, get_angle_complex_samples, get_angle_power_samples
 
 
@@ -128,6 +129,7 @@ def add_depth(
     return ds
 
 
+@add_processing_level("L2A")
 def add_location(ds: xr.Dataset, echodata: EchoData = None, nmea_sentence: Optional[str] = None):
     """
     Add geographical location (latitude/longitude) to the Sv dataset.
@@ -147,28 +149,29 @@ def add_location(ds: xr.Dataset, echodata: EchoData = None, nmea_sentence: Optio
 
     Returns
     -------
-    The input dataset with the the location data added
+    The input dataset with the location data added
     """
 
     def sel_interp(var):
         # NMEA sentence selection
         if nmea_sentence:
-            coord_var = echodata["Platform"][var][
+            position_var = echodata["Platform"][var][
                 echodata["Platform"]["sentence_type"] == nmea_sentence
             ]
         else:
-            coord_var = echodata["Platform"][var]
+            position_var = echodata["Platform"][var]
 
-        if len(coord_var) == 1:
+        if len(position_var) == 1:
             # Propagate single, fixed-location coordinate
             return xr.DataArray(
-                data=coord_var.values[0] * np.ones(len(ds["ping_time"]), dtype=np.float64),
+                data=position_var.values[0] * np.ones(len(ds["ping_time"]), dtype=np.float64),
                 dims=["ping_time"],
-                attrs=coord_var.attrs,
+                attrs=position_var.attrs,
             )
         else:
             # Interpolation. time1 is always associated with location data
-            return coord_var.interp(time1=ds["ping_time"])
+            # Values may be nan if there are ping_time values outside the time1 range
+            return position_var.interp(time1=ds["ping_time"])
 
     if "longitude" not in echodata["Platform"] or echodata["Platform"]["longitude"].isnull().all():
         raise ValueError("Coordinate variables not present or all nan")
@@ -296,19 +299,19 @@ def add_splitbeam_angle(
         source_Sv = xr.open_dataset(source_Sv, engine=file_type, chunks={}, **storage_options)
 
     # raise not implemented error if source_Sv corresponds to MVBS
-    if source_Sv.attrs["processing_function"] == "preprocess.compute_MVBS":
+    if source_Sv.attrs["processing_function"] == "commongrid.compute_MVBS":
         raise NotImplementedError("Adding split-beam data to MVBS has not been implemented!")
 
     # check that the appropriate waveform and encode mode have been given
     # and obtain the echodata group path corresponding to encode_mode
-    encode_mode_ed_group = retrieve_correct_beam_group(echodata, waveform_mode, encode_mode)
+    ed_beam_group = retrieve_correct_beam_group(echodata, waveform_mode, encode_mode)
 
     # check that source_Sv at least has a channel dimension
     if "channel" not in source_Sv.variables:
         raise ValueError("The input source_Sv Dataset must have a channel dimension!")
 
     # Select ds_beam channels from source_Sv
-    ds_beam = echodata[encode_mode_ed_group].sel(channel=source_Sv["channel"].values)
+    ds_beam = echodata[ed_beam_group].sel(channel=source_Sv["channel"].values)
 
     # Assemble angle param dict
     angle_param_list = [
