@@ -16,6 +16,7 @@ from ..echodata.echodata import XARRAY_ENGINE_MAP, EchoData
 from ..utils import io
 from ..utils.coding import COMPRESSION_SETTINGS
 from ..utils.log import _init_logger
+from ..utils.prov import add_processing_level
 
 BEAM_SUBGROUP_DEFAULT = "Beam_group1"
 
@@ -306,6 +307,7 @@ def _check_file(
     return str(raw_file), str(xml)
 
 
+@add_processing_level("L1A", is_echodata=True)
 def open_raw(
     raw_file: "PathHint",
     sonar_model: "SonarModelsHint",
