@@ -194,6 +194,7 @@ class SetGroupsEK80(SetGroupsBase):
             "transducer_frequency",
             "serial_number",
             "transducer_name",
+            "transducer_serial_number",
             "application_name",
             "application_version",
             "channel_id_short",
@@ -236,17 +237,27 @@ class SetGroupsEK80(SetGroupsBase):
                     "standard_name": "sound_frequency",
                 },
             ),
-            "serial_number": (["channel"], var["serial_number"]),
-            "transducer_name": (["channel"], var["transducer_name"]),
-            "sonar_serial_number": (["channel"], var["channel_id_short"]),
-            "sonar_software_name": (
+            "transceiver_serial_number": (
                 ["channel"],
-                var["application_name"],
-            ),  # identical for all channels
-            "sonar_software_version": (
+                var["serial_number"],
+                {
+                    "long_name": "Transceiver serial number",
+                },
+            ),
+            "transducer_name": (
                 ["channel"],
-                var["application_version"],
-            ),  # identical for all channels
+                var["transducer_name"],
+                {
+                    "long_name": "Transducer name",
+                },
+            ),
+            "transducer_serial_number": (
+                ["channel"],
+                var["transducer_serial_number"],
+                {
+                    "long_name": "Transducer serial number",
+                },
+            ),
         }
         ds = xr.Dataset(
             {**sonar_vars, **beam_groups_vars},
@@ -264,6 +275,12 @@ class SetGroupsEK80(SetGroupsBase):
         sonar_attr_dict = {
             "sonar_manufacturer": "Simrad",
             "sonar_model": self.sonar_model,
+            # transducer (sonar) serial number is not reliably stored in the EK80 raw
+            # data file and would be channel-dependent. For consistency with EK60,
+            # will not try to populate sonar_serial_number from the raw datagrams
+            "sonar_serial_number": "",
+            "sonar_software_name": var["application_name"][0],
+            "sonar_software_version": var["application_version"][0],
             "sonar_type": "echosounder",
         }
         ds = ds.assign_attrs(sonar_attr_dict)
