@@ -7,7 +7,10 @@ from .echodata import EchoData
 
 
 def open_converted(
-    converted_raw_path: "PathHint", storage_options: Dict[str, str] = None, **kwargs
+    converted_raw_path: "PathHint",
+    storage_options: Dict[str, str] = None,
+    **kwargs
+    # kwargs: Dict[str, Any] = {'chunks': 'auto'} # TODO: do we need this?
 ):
     """Create an EchoData object from a single converted netcdf or zarr file.
 
